@@ -33,48 +33,48 @@
 
 #include "initialization.h"
 
-#include "simulation_data.h"
-#include "util/omc_error.h"
-#include "openmodelica.h"
-#include "openmodelica_func.h"
-#include "simulation/options.h"
-#include "simulation/solver/model_help.h"
+#include "../../../util/omc_error.h"
+#include "../../../util/omc_file.h"
+#include "../../../openmodelica.h"
+#include "../../../openmodelica_func.h"
+#include "../../../simulation/options.h"
+#include "../model_help.h"
 #if !defined(OMC_MINIMAL_RUNTIME)
-#include "util/read_matlab4.h"
+#include "../../../util/read_matlab4.h"
 #endif
-#include "simulation/solver/events.h"
-#include "simulation/solver/stateset.h"
-#include "meta/meta_modelica.h"
+#include "../events.h"
+#include "../stateset.h"
+#include "../../../meta/meta_modelica.h"
 
 #if defined(OMC_NUM_MIXED_SYSTEMS) && OMC_NUM_MIXED_SYSTEMS==0
 #define check_mixed_solutions(X,Y) 0
 #else
-#include "simulation/solver/mixedSystem.h"
+#include "../mixedSystem.h"
 #endif
 
 #if defined(OMC_NUM_LINEAR_SYSTEMS) && OMC_NUM_LINEAR_SYSTEMS==0
 #define check_linear_solutions(X,Y) 0
 #define updateStaticDataOfLinearSystems(X,Y)
 #else
-#include "simulation/solver/linearSystem.h"
+#include "../linearSystem.h"
 #endif
 
 #if defined(OMC_NUM_NONLINEAR_SYSTEMS) && OMC_NUM_NONLINEAR_SYSTEMS==0
 #define check_nonlinear_solutions(X,Y) 0
 #define updateStaticDataOfNonlinearSystems(X,Y)
 #else
-#include "simulation/solver/nonlinearSystem.h"
+#include "../nonlinearSystem.h"
 #endif
 
-#include "simulation/solver/delay.h"
-#include "simulation/solver/synchronous.h"
+#include "../delay.h"
+#include "../synchronous.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
-int init_lambda_steps = 4;
+extern int init_lambda_steps = 3;
 
 /*! \fn void dumpInitializationStatus(DATA *data)
  *
@@ -89,8 +89,8 @@ void dumpInitialSolution(DATA *simData)
   const MODEL_DATA      *mData = simData->modelData;
   const SIMULATION_INFO *sInfo = simData->simulationInfo;
 
-  if (ACTIVE_STREAM(LOG_INIT))
-    printParameters(simData, LOG_INIT);
+  if (ACTIVE_STREAM(LOG_INIT_V))
+    printParameters(simData, LOG_INIT_V);
 
   if (!ACTIVE_STREAM(LOG_SOTI)) return;
   infoStreamPrint(LOG_SOTI, 1, "### SOLUTION OF THE INITIALIZATION ###");
@@ -177,6 +177,7 @@ void dumpInitialSolution(DATA *simData)
  *  \param [ref] [threadData]
  *
  *  \author lochel
+ *  \author ptaeuber
  */
 static int symbolic_initialization(DATA *data, threadData_t *threadData)
 {
@@ -185,6 +186,33 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
   FILE *pFile = NULL;
   long i;
   MODEL_DATA *mData = data->modelData;
+  int homotopySupport = 0;
+  int solveWithGlobalHomotopy;
+  int adaptiveGlobal;
+  int kinsol = 0;
+
+#if !defined(OMC_MINIMAL_RUNTIME)
+  kinsol = (data->simulationInfo->nlsMethod == NLS_KINSOL);
+#endif
+
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
+  for(i=0; i<mData->nNonLinearSystems; i++) {
+    if (data->simulationInfo->nonlinearSystemData[i].homotopySupport) {
+      homotopySupport = 1;
+      break;
+    }
+  }
+#endif
+  /* useHomotopy=1: global homotopy (equidistant lambda) */
+  if (data->callback->useHomotopy == 1 && omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] != 1 && omc_flag[FLAG_NO_HOMOTOPY_ON_FIRST_TRY] != 1) {
+      omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] = 1;
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Model contains homotopy operator: Use adaptive homotopy method to solve initialization problem. "
+                                            "To disable initialization with homotopy operator use \"-noHomotopyOnFirstTry\".");
+  }
+
+  adaptiveGlobal = data->callback->useHomotopy == 2;  /* new global homotopy approach (adaptive lambda) */
+  solveWithGlobalHomotopy = homotopySupport
+                            && ((data->callback->useHomotopy == 1 && init_lambda_steps >= 1) || adaptiveGlobal);
 
 #if !defined(OMC_NDELAY_EXPRESSIONS) || OMC_NDELAY_EXPRESSIONS>0
   /* initial sample and delay before initial the system */
@@ -196,98 +224,190 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
 
   /* If there is no homotopy in the model or local homotopy is activated
      or homotopy is disabled by runtime flag '-ils=<lambda_steps>',
-     solve WITHOUT HOMOTOPY. */
-  if (data->callback->useHomotopy == 0 || init_lambda_steps < 2){
+     solve WITHOUT HOMOTOPY or LOCAL HOMOTOPY. */
+  if (!solveWithGlobalHomotopy){
     data->simulationInfo->lambda = 1.0;
     data->callback->functionInitialEquations(data, threadData);
-    infoStreamPrint(LOG_STDOUT, 0, "The initialization finished successfully without homotopy method.");
 
   /* If there is homotopy in the model and global homotopy is activated
      and homotopy on first try is deactivated,
-     TRY TO SOLVE WITHOUT HOMOTOPY FIRST. */
+     TRY TO SOLVE WITHOUT HOMOTOPY FIRST.
+     TO-DO: For the adaptive global approach, provide a separate DAE with
+     the original unmanipulated systems for trying without homotopy */
   } else if (!omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY]) {
     /* try */
 #ifndef OMC_EMCC
   MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
-
-    data->simulationInfo->lambda = 1.0;
-    infoStreamPrint(LOG_INIT, 0, "Try to solve the initialization problem without homotopy first.");
-    data->callback->functionInitialEquations(data, threadData);
-    init_lambda_steps = 0;
-    infoStreamPrint(LOG_STDOUT, 0, "The initialization finished successfully without homotopy method.");
+    if (adaptiveGlobal && kinsol) {
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Automatically set -homotopyOnFirstTry, because trying without homotopy first is not supported for the adaptive global approach in combination with KINSOL.");
+    } else {
+      if (adaptiveGlobal)
+        data->callback->useHomotopy = 1;  /* global homotopy (equidistant lambda) */
+      data->simulationInfo->lambda = 1.0;
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Try to solve the initialization problem without homotopy first.");
+      data->callback->functionInitialEquations(data, threadData);
+      solveWithGlobalHomotopy = 0;
+    }
 
     /* catch */
 #ifndef OMC_EMCC
   MMC_CATCH_INTERNAL(simulationJumpBuffer)
 #endif
-    if(init_lambda_steps > 0)
-      warningStreamPrint(LOG_ASSERT, 0, "Failed to solve the initial system without homotopy method. If homotopy is available the homotopy method is used now.");
+    if (adaptiveGlobal)
+      data->callback->useHomotopy = 2; /* new global homotopy approach (adaptive lambda) */
+    if(solveWithGlobalHomotopy) {
+      if (!kinsol)
+        warningStreamPrint(LOG_ASSERT, 0, "Failed to solve the initialization problem without homotopy method. If homotopy is available the homotopy method is used now.");
+      omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] = 1;
+      setAllParamsToStart(data);
+      setAllVarsToStart(data);
+      data->callback->updateBoundParameters(data, threadData);
+      data->callback->updateBoundVariableAttributes(data, threadData);
+    }
   }
 
-  /* If there is homotopy in the model and global homotopy is activated
+  /* If there is homotopy in the model and the equidistant global homotopy approach is activated
      and solving without homotopy failed or is not wanted,
-     use GLOBAL HOMOTOPY METHOD. */
-  if (data->callback->useHomotopy == 1 && init_lambda_steps > 1)
+     use EQUIDISTANT GLOBAL HOMOTOPY METHOD. */
+  if (data->callback->useHomotopy == 1 && solveWithGlobalHomotopy)
   {
     long step;
     char buffer[4096];
+    double lambda;
+    int success = 0;
+
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Global homotopy with equidistant step size started.");
 
 #if !defined(OMC_NO_FILESYSTEM)
-    if(ACTIVE_STREAM(LOG_INIT))
+    const char sep[] = ",";
+    if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
     {
-      sprintf(buffer, "%s_global_homotopy.csv", mData->modelFilePrefix);
-      pFile = fopen(buffer, "wt");
-      fprintf(pFile, "\"sep=,\"\n%s", "lambda");
-      for(i=0; i<mData->nVariablesReal; ++i)
-        fprintf(pFile, ",%s", mData->realVarsData[i].info.name);
+      if (omc_flag[FLAG_OUTPUT_PATH]) { /* Add output path to file name */
+        sprintf(buffer, "%s/%s_equidistant_global_homotopy.csv", omc_flagValue[FLAG_OUTPUT_PATH], mData->modelFilePrefix);
+      }
+      else
+      {
+        sprintf(buffer, "%s_equidistant_global_homotopy.csv", mData->modelFilePrefix);
+      }
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "The homotopy path will be exported to %s.", buffer);
+      pFile = omc_fopen(buffer, "wt");
+      if (pFile == NULL)
+      {
+        throwStreamPrint(threadData, "Could not write to `%s`.", buffer);
+      }
+      fprintf(pFile, "\"lambda\"");
+      for(i=0; i<mData->nVariablesReal; ++i) {
+        fprintf(pFile, "%s\"%s\"", sep, mData->realVarsData[i].info.name);
+      }
       fprintf(pFile, "\n");
     }
 #endif
 
-    infoStreamPrint(LOG_INIT, 1, "homotopy process\n---------------------------");
-    for(step=0; step<init_lambda_steps; ++step)
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 1, "homotopy process\n---------------------------");
+    /* try */
+#ifndef OMC_EMCC
+  MMC_TRY_INTERNAL(simulationJumpBuffer)
+#endif
+    for(step=0; step<=init_lambda_steps; ++step)
     {
-      data->simulationInfo->lambda = ((double)step)/(init_lambda_steps-1);
-      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g", data->simulationInfo->lambda);
+      data->simulationInfo->lambda = ((double)step)/(init_lambda_steps);
+      lambda = data->simulationInfo->lambda;
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "homotopy parameter lambda = %g", lambda);
 
-      if(data->simulationInfo->lambda > 1.0) {
+      if(data->simulationInfo->lambda > 1.0)
+      {
         data->simulationInfo->lambda = 1.0;
+        lambda = 1.0;
       }
 
       if(0 == step)
-        data->callback->functionInitialEquations_lambda0(data, threadData);
+      {
+        if(data->callback->functionInitialEquations_lambda0 != NULL)
+        {
+          data->callback->functionInitialEquations_lambda0(data, threadData);
+        }
+        else
+        {
+          warningStreamPrint(LOG_INIT_HOMOTOPY, 0, "No initialEquation_lambda0 was generated. Using normal initial equation system with lambda=0 instead.");
+          data->callback->functionInitialEquations(data, threadData);
+        }
+      }
       else
+      {
         data->callback->functionInitialEquations(data, threadData);
+      }
 
-      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g done\n---------------------------", data->simulationInfo->lambda);
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "homotopy parameter lambda = %g done\n---------------------------", lambda);
 
 #if !defined(OMC_NO_FILESYSTEM)
-      if(ACTIVE_STREAM(LOG_INIT))
+      if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
       {
-        fprintf(pFile, "%.16g", data->simulationInfo->lambda);
+        fprintf(pFile, "%.16g", lambda);
         for(i=0; i<mData->nVariablesReal; ++i)
-          fprintf(pFile, ",%.16g", data->localData[0]->realVars[i]);
+        {
+          fprintf(pFile, "%s%.16g", sep, data->localData[0]->realVars[i]);
+        }
         fprintf(pFile, "\n");
       }
 #endif
-
-      if (check_nonlinear_solutions(data, 0) ||
-          check_linear_solutions(data, 0) ||
-          check_mixed_solutions(data, 0))
-        break;
     }
-    infoStreamPrint(LOG_STDOUT, 0, "The initialization finished successfully with homotopy method.");
-    messageClose(LOG_INIT);
+    success = 1;
+    /* catch */
+#ifndef OMC_EMCC
+  MMC_CATCH_INTERNAL(simulationJumpBuffer)
+#endif
+    /* Error handling in case an assert was thrown */
+    if (!success)
+    {
+      messageClose(LOG_INIT_HOMOTOPY);
+#if !defined(OMC_NO_FILESYSTEM)
+      if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
+        fclose(pFile);
+#endif
+      errorStreamPrint(LOG_ASSERT, 0, "Failed to solve the initialization problem with global homotopy with equidistant step size.");
+      throwStreamPrint(threadData, "Unable to solve initialization problem.");
+    }
+
+    data->simulationInfo->homotopySteps += init_lambda_steps;
+    messageClose(LOG_INIT_HOMOTOPY);
+
+#if !defined(OMC_NO_FILESYSTEM)
+    if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
+      fclose(pFile);
+#endif
+  }
+
+  /* If there is homotopy in the model and the adaptive global homotopy approach is activated
+     and solving without homotopy failed or is not wanted,
+     use ADAPTIVE GLOBAL HOMOTOPY APPROACH. */
+  if (adaptiveGlobal && solveWithGlobalHomotopy)
+  {
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Global homotopy with adaptive step size started.");
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 1, "homotopy process\n---------------------------");
+
+    // Solve lambda0-DAE
+    data->simulationInfo->lambda = 0;
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "solve simplified lambda0-DAE");
+    if(data->callback->functionInitialEquations_lambda0 != NULL)
+    {
+      data->callback->functionInitialEquations_lambda0(data, threadData);
+    }
+    else
+    {
+      warningStreamPrint(LOG_INIT_HOMOTOPY, 0, "No initialEquation_lambda0 was generated. Using normal initial equation system with lambda=0 instead.");
+      data->callback->functionInitialEquations(data, threadData);
+    }
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "solving simplified lambda0-DAE done\n---------------------------");
+
+    // Run along the homotopy path and solve the actual system
+    data->callback->functionInitialEquations(data, threadData);
+
+    messageClose(LOG_INIT_HOMOTOPY);
   }
 
   storeRelations(data);
 
-#if !defined(OMC_NO_FILESYSTEM)
-  if(data->callback->useHomotopy == 1 && init_lambda_steps > 1 && ACTIVE_STREAM(LOG_INIT))
-    fclose(pFile);
-#endif
-
   /* check for over-determined systems */
   retVal = data->callback->functionRemovedInitialEquations(data, threadData);
 
@@ -387,13 +507,10 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
   }
 
   pError = omc_new_matlab4_reader(pInitFile, &reader);
-  if(pError)
-  {
+  if(pError) {
     throwStreamPrint(threadData, "unable to read input-file <%s> [%s]", pInitFile, pError);
     return 1;
-  }
-  else
-  {
+  } else {
     infoStreamPrint(LOG_INIT, 0, "import real variables");
     for(i=0; i<mData->nVariablesReal; ++i) {
       pVar = omc_matlab4_find_var(&reader, mData->realVarsData[i].info.name);
@@ -403,10 +520,9 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         pVar = omc_matlab4_find_var(&reader, newVarname);
         free(newVarname);
       }
-
       if(pVar) {
         omc_matlab4_val(&(mData->realVarsData[i].attribute.start), &reader, pVar, initTime);
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realVarsData[i].info.name, mData->realVarsData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realVarsData[i].info.name, mData->realVarsData[i].attribute.start);
       } else if((strlen(mData->realVarsData[i].info.name) > 0) &&
               (mData->realVarsData[i].info.name[0] != '$') &&
               (strncmp(mData->realVarsData[i].info.name, "der($", 5) != 0)) {
@@ -428,7 +544,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
       if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
         data->simulationInfo->realParameter[i] = mData->realParameterData[i].attribute.start;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
@@ -446,7 +562,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
 
       if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
@@ -467,7 +583,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->integerParameterData[i].attribute.start = (modelica_integer)value;
         data->simulationInfo->integerParameter[i] = (modelica_integer)value;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%ld)", mData->integerParameterData[i].info.name, mData->integerParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%ld)", mData->integerParameterData[i].info.name, mData->integerParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import integer parameter %s from given file", mData->integerParameterData[i].info.name);
       }
@@ -487,7 +603,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->booleanParameterData[i].attribute.start = (modelica_boolean)value;
         data->simulationInfo->booleanParameter[i] = (modelica_boolean)value;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%s)", mData->booleanParameterData[i].info.name, mData->booleanParameterData[i].attribute.start ? "true" : "false");
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%s)", mData->booleanParameterData[i].info.name, mData->booleanParameterData[i].attribute.start ? "true" : "false");
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import boolean parameter %s from given file", mData->booleanParameterData[i].info.name);
       }
@@ -551,9 +667,12 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   int retVal = -1;
   int i;
 
+  data->simulationInfo->homotopySteps = 0;
+
   infoStreamPrint(LOG_INIT, 0, "### START INITIALIZATION ###");
 
-  setAllParamsToStart(data);
+  if (strcmp(pInitMethod, "fmi"))
+    setAllParamsToStart(data);
 
 #if !defined(OMC_MINIMAL_RUNTIME)
   /* import start values from extern mat-file */
@@ -569,7 +688,8 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   }
 #endif
   /* set up all variables with their start-values */
-  setAllVarsToStart(data);
+  if (strcmp(pInitMethod, "fmi"))
+    setAllVarsToStart(data);
 
   if(!(pInitFile && strcmp(pInitFile, ""))) {
     data->callback->updateBoundParameters(data, threadData);
@@ -581,7 +701,7 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   updateStaticDataOfNonlinearSystems(data, threadData);
 
   /* if there are user-specified options, use them! */
-  if (pInitMethod && strcmp(pInitMethod, "")) {
+  if (pInitMethod && (strcmp(pInitMethod, "") && strcmp(pInitMethod, "fmi"))) {
     initMethod = IIM_UNKNOWN;
 
     for (i=1; i<IIM_MAX; ++i) {
@@ -679,6 +799,9 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   printZeroCrossings(data, LOG_EVENTS);
 #endif
 
+  /* Check for warning of variables out of range assert(min<x || x>xmax, ...)*/
+  data->callback->checkForAsserts(data, threadData);
+
   /* valid system for the first time! */
   TRACE_POP
   return retVal;
