@@ -88,6 +88,7 @@ class LibraryTreeItem;
 class GitCommands;
 class CommitChangesDialog;
 class TraceabilityInformationURI;
+class StatusBar;
 class TraceabilityGraphViewWidget;
 class SearchWidget;
 
@@ -95,7 +96,7 @@ class MainWindow : public QMainWindow
 {
   Q_OBJECT
 public:
-  enum { MaxRecentFiles = 8 };
+  enum { MaxRecentModels = 8 };
 private:
   MainWindow(QWidget *parent = 0);
   static MainWindow *mpInstance;
@@ -137,7 +138,7 @@ public:
   GitCommands* getGitCommands() {return mpGitCommands;}
   CommitChangesDialog* getCommitChangesDialog() {return mpCommitChangesDialog;}
   TraceabilityInformationURI* getTraceabilityInformationURI() {return mpTraceabilityInformationURI;}
-  QStatusBar* getStatusBar() {return mpStatusBar;}
+  StatusBar* getStatusBar() {return mpStatusBar;}
   QProgressBar* getProgressBar() {return mpProgressBar;}
   void showProgressBar() {mpProgressBar->setVisible(true);}
   void hideProgressBar() {mpProgressBar->setVisible(false);}
@@ -160,7 +161,6 @@ public:
   QAction* getCloseAllWindowsAction() {return mpCloseAllWindowsAction;}
   QAction* getCloseAllWindowsButThisAction() {return mpCloseAllWindowsButThisAction;}
   QAction* getSimulationSetupAction() {return mpSimulationSetupAction;}
-  QAction* getOMSInstantiateModelAction() {return mpOMSInstantiateModelAction;}
   QAction* getSimulateModelAction() {return mpSimulateModelAction;}
   QAction* getSimulateWithTransformationalDebuggerAction() {return mpSimulateWithTransformationalDebuggerAction;}
   QAction* getSimulateWithAlgorithmicDebuggerAction() {return mpSimulateWithAlgorithmicDebuggerAction;}
@@ -213,7 +213,8 @@ public:
   QToolBar* getTLMSimulationToolbar() const {return mpTLMSimulationToolbar;}
   QToolBar* getOMSimulatorToobar() const {return mpOMSimulatorToobar;}
   void addRecentFile(const QString &fileName, const QString &encoding);
-  void updateRecentFileActions();
+  void updateRecentFileActionsAndList();
+  void createRecentFileActions();
   void closeEvent(QCloseEvent *event) override;
   int askForExit();
   void beforeClosingMainWindow();
@@ -226,7 +227,6 @@ public:
   void simulateWithAnimation(LibraryTreeItem *pLibraryTreeItem);
 #endif
   void simulationSetup(LibraryTreeItem *pLibraryTreeItem);
-  void instantiateOMSModel(LibraryTreeItem *pLibraryTreeItem, bool checked);
   void instantiateModel(LibraryTreeItem *pLibraryTreeItem);
   void checkModel(LibraryTreeItem *pLibraryTreeItem);
   void checkAllModels(LibraryTreeItem *pLibraryTreeItem);
@@ -298,7 +298,7 @@ private:
   QProgressBar *mpProgressBar;
   Label *mpPositionLabel;
   QTabBar *mpPerspectiveTabbar;
-  QStatusBar *mpStatusBar;
+  StatusBar *mpStatusBar;
   QTimer *mpAutoSaveTimer;
   QShortcut *mpSearchBrowserShortcut;
   // File Menu
@@ -334,7 +334,6 @@ private:
   QAction *mpExportXMLAction;
   QAction *mpExportFigaroAction;
   QAction *mpExportToOMNotebookAction;
-  QAction *mpRecentFileActions[MaxRecentFiles];
   QAction *mpClearRecentFilesAction;
   QAction *mpPrintModelAction;
   QAction *mpQuitAction;
@@ -359,7 +358,6 @@ private:
   QAction *mpCheckAllModelsAction;
   QAction *mpInstantiateModelAction;
   QAction *mpSimulationSetupAction;
-  QAction *mpOMSInstantiateModelAction;
   QAction *mpSimulateModelAction;
   QAction *mpSimulateWithTransformationalDebuggerAction;
   QAction *mpSimulateWithAlgorithmicDebuggerAction;
@@ -381,6 +379,7 @@ private:
   QAction *mpShowOMCLoggerWidgetAction;
   QAction *mpShowOpenModelicaCommandPromptAction;
   QAction *mpShowOMCDiffWidgetAction;
+  QAction *mpOpenTemporaryDirectoryAction;
   QAction *mpOpenWorkingDirectoryAction;
   QAction *mpOpenTerminalAction;
   QAction *mpOptionsAction;
@@ -408,7 +407,7 @@ private:
   QAction *mpConnectModeAction;
   QAction *mpTransitionModeAction;
   // Model Switcher Toolbar Actions
-  QAction *mpModelSwitcherActions[MaxRecentFiles];
+  QAction *mpModelSwitcherActions[MaxRecentModels];
   // Plot Toolbar Actions
   QAction *mpReSimulateModelAction;
   QAction *mpReSimulateSetupAction;
@@ -499,7 +498,6 @@ public slots:
   void checkAllModels();
   void instantiateModel();
   void openSimulationDialog();
-  void instantiateOMSModel(bool checked);
   void simulateModel();
   void simulateModelWithTransformationalDebugger();
   void simulateModelWithAlgorithmicDebugger();
@@ -520,6 +518,7 @@ public slots:
   void exportToClipboard();
   void fetchInterfaceData();
   void TLMSimulate();
+  void openTemporaryDirectory();
   void openWorkingDirectory();
   void openTerminal();
   void openConfigurationOptions();
@@ -573,11 +572,16 @@ protected:
   virtual void dropEvent(QDropEvent *event) override;
 };
 
+class QNetworkReply;
 class AboutOMEditDialog : public QDialog
 {
   Q_OBJECT
 public:
   AboutOMEditDialog(MainWindow *pMainWindow);
+private:
+  Label *mpOMContributorsLabel;
+private slots:
+  void readOMContributors(QNetworkReply *pNetworkReply);
 };
 
 #endif // MAINWINDOW_H
