@@ -165,6 +165,53 @@ oms_status_enu_t oms::Model::loadSnapshot(const char* snapshot)
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::Model::importSnapshot(const char* snapshot)
+{
+  if (!validState(oms_modelState_virgin))
+    return logError_ModelInWrongState(this);
+
+  pugi::xml_document doc;
+  pugi::xml_parse_result result = doc.load(snapshot);
+  if (!result)
+    return logError("loading snapshot failed (" + std::string(result.description()) + ")");
+
+  snapShot = doc.document_element(); // oms:snapshot
+
+  // get ssd:SystemStructureDescription
+  pugi::xml_node ssdNode = snapShot.child(oms::ssp::Version1_0::ssd_file).child(oms::ssp::Draft20180219::ssd::system_structure_description);
+
+  ComRef new_cref = ComRef(ssdNode.attribute("name").as_string());
+  std::string ssdVersion = ssdNode.attribute("version").as_string();
+
+  if (new_cref != getCref())
+    return logError("this API cannot be used to rename a model");
+
+  if (ssdVersion != "Draft20180219" && ssdVersion != "1.0")
+    logWarning("Unknown SSD version: " + ssdVersion);
+
+  System* old_root_system = system;
+  system = NULL;
+
+  bool old_copyResources = copyResources();
+  copyResources(false);
+  oms_status_enu_t status = importFromSSD(ssdNode);
+  copyResources(old_copyResources);
+
+  if (oms_status_ok != status)
+  {
+    system = old_root_system;
+    return logError("loading snapshot failed");
+  }
+
+  if (old_root_system)
+  {
+    delete old_root_system;
+    old_root_system = NULL;
+  }
+
+  return oms_status_ok;
+}
+
 oms::System* oms::Model::getSystem(const oms::ComRef& cref)
 {
   if (!system)
@@ -288,6 +335,162 @@ oms_status_enu_t oms::Model::list(const oms::ComRef& cref, char** contents)
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::Model::exportSnapshot(const oms::ComRef& cref, char** contents)
+{
+  // only top level model is allowed
+  if (!cref.isEmpty())
+  {
+    //return logError("only top level model is allowed, unknown model: " + std::string(cref));
+    return logError("\"" + std::string(getCref()+std::string(cref)) + "\" is not a top level model");
+  }
+
+  struct xmlStringWriter : pugi::xml_writer
+  {
+    std::string result;
+    virtual void write(const void* data, size_t size)
+    {
+      result += std::string(static_cast<const char*>(data), size);
+    }
+  };
+
+  xmlStringWriter writer;
+  pugi::xml_document doc;
+  pugi::xml_document ssvdoc;
+
+  // generate XML declaration for ssv file
+  pugi::xml_node ssvDeclarationNode = ssvdoc.append_child(pugi::node_declaration);
+  ssvDeclarationNode.append_attribute("version") = "1.0";
+  ssvDeclarationNode.append_attribute("encoding") = "UTF-8";
+
+  pugi::xml_node node_parameterset = ssvdoc.append_child(oms::ssp::Version1_0::ssv::parameter_set);
+  node_parameterset.append_attribute("xmlns:ssc") = "http://ssp-standard.org/SSP1/SystemStructureCommon";
+  node_parameterset.append_attribute("xmlns:ssv") = "http://ssp-standard.org/SSP1/SystemStructureParameterValues";
+  node_parameterset.append_attribute("version") = "1.0";
+  node_parameterset.append_attribute("name") = "parameters";
+  pugi::xml_node node_parameters = node_parameterset.append_child(oms::ssp::Version1_0::ssv::parameters);
+
+  // list model
+  pugi::xml_node snapshotnode = doc.append_child(oms::ssp::Version1_0::snap_shot);
+  pugi::xml_node ssdfilenode = snapshotnode.append_child(oms::ssp::Version1_0::ssd_file);
+  ssdfilenode.append_attribute("name") = "SystemStructure.ssd";
+
+  pugi::xml_node node = ssdfilenode.append_child(oms::ssp::Draft20180219::ssd::system_structure_description);
+  exportToSSD(node, node_parameters);
+
+  // update ssv file if exist
+  if (!Flags::ExportParametersInline())
+  {
+    // update parameterBindings in ssd
+    pugi::xml_node system_node = node.child(oms::ssp::Draft20180219::ssd::system);
+    updateParameterBindingsToSSD(system_node, node_parameters, true);
+
+    // update <oms:ssv_file> after </oms:ssd_file>
+    pugi::xml_node last = doc.last_child();
+    pugi::xml_node ssvfilenode  = last.append_child(oms::ssp::Version1_0::ssv_file);
+    std::string ssvFilePath = "resources/" + std::string(this->getCref()) + ".ssv";
+    ssvfilenode.append_attribute("name") = ssvFilePath.c_str();
+    // dump all the ssv file contents
+    ssvfilenode.append_copy(node_parameterset);
+    // TODO ssm file
+  }
+
+  doc.save(writer);
+  *contents = (char*) malloc(strlen(writer.result.c_str()) + 1);
+  strcpy(*contents, writer.result.c_str());
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::Model::exportSSVTemplate(const oms::ComRef& cref, const std::string& filename)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  pugi::xml_document ssvdoc;
+
+  std::string extension = "";
+  if (filename.length() > 4)
+    extension = filename.substr(filename.length() - 4);
+
+  if (extension != ".ssv")
+    return logError("filename extension must be \".ssv\"; no other formats are supported");
+
+  // generate XML declaration for ssv file
+  pugi::xml_node ssvDeclarationNode = ssvdoc.append_child(pugi::node_declaration);
+  ssvDeclarationNode.append_attribute("version") = "1.0";
+  ssvDeclarationNode.append_attribute("encoding") = "UTF-8";
+
+  pugi::xml_node node_parameterset = ssvdoc.append_child(oms::ssp::Version1_0::ssv::parameter_set);
+  node_parameterset.append_attribute("xmlns:ssc") = "http://ssp-standard.org/SSP1/SystemStructureCommon";
+  node_parameterset.append_attribute("xmlns:ssv") = "http://ssp-standard.org/SSP1/SystemStructureParameterValues";
+
+  node_parameterset.append_attribute("version") = "1.0";
+  node_parameterset.append_attribute("name") = "modelDescriptionStartValues";
+  pugi::xml_node node_parameters = node_parameterset.append_child(oms::ssp::Version1_0::ssv::parameters);
+
+  for (const auto& component : system->getComponents())
+  {
+    if(component.first == tail || cref.isEmpty()) // allow querying single component or whole model
+    {
+      if (oms_status_ok != component.second->exportToSSVTemplate(node_parameters))
+      {
+        return logError("export of ssv template failed for component " + std::string(system->getFullCref()+std::string(component.first)));
+      }
+    }
+  }
+
+  //ssvdoc.save(std::cout);
+
+  if (!ssvdoc.save_file(filename.c_str()))
+    return logError("failed to export  \"" + filename + "\" (for model \"" + std::string(this->getCref()) + "\")");
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::Model::exportSSMTemplate(const oms::ComRef& cref, const std::string& filename)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  pugi::xml_document ssmdoc;
+
+  std::string extension = "";
+  if (filename.length() > 4)
+    extension = filename.substr(filename.length() - 4);
+
+  if (extension != ".ssm")
+    return logError("filename extension must be \".ssm\"; no other formats are supported");
+
+  // generate XML declaration for ssv file
+  pugi::xml_node ssmDeclarationNode = ssmdoc.append_child(pugi::node_declaration);
+  ssmDeclarationNode.append_attribute("version") = "1.0";
+  ssmDeclarationNode.append_attribute("encoding") = "UTF-8";
+
+  pugi::xml_node node_parameterMapping = ssmdoc.append_child(oms::ssp::Version1_0::ssm::parameter_mapping);
+  node_parameterMapping.append_attribute("xmlns:ssc") = "http://ssp-standard.org/SSP1/SystemStructureCommon";
+  node_parameterMapping.append_attribute("xmlns:ssm") = "http://ssp-standard.org/SSP1/SystemStructureParameterMapping";
+
+  node_parameterMapping.append_attribute("version") = "1.0";
+
+  for (const auto& component : system->getComponents())
+  {
+    if(component.first == tail || cref.isEmpty()) // allow querying single component or whole model
+    {
+      if (oms_status_ok != component.second->exportToSSMTemplate(node_parameterMapping))
+      {
+        return logError("export of ssm template failed for component " + std::string(system->getFullCref()+std::string(component.first)));
+      }
+    }
+  }
+
+  //ssmdoc.save(std::cout);
+
+  if (!ssmdoc.save_file(filename.c_str()))
+    return logError("failed to export  \"" + filename + "\" (for model \"" + std::string(this->getCref()) + "\")");
+
+  return oms_status_ok;
+}
+
 /*
  * This function update the ParameterBindings in SSD, to link with a SSV file, (e.g)
  * <ssd:ParameterBindings>
@@ -304,9 +507,9 @@ oms_status_enu_t oms::Model::updateParameterBindingsToSSD(pugi::xml_node& node,
     // update the ssd with the top level parameterBindings (e.g)  <ParameterBinding source="resources/ControlledTemperature.ssv">
     for(pugi::xml_node_iterator it = node.begin(); it != node.end(); ++it)
     {
-      if (std::string(it->name()) == oms::ssp::Draft20180219::ssd::connectors) // insert the parameter bindings after top-level connectors node
+      if (std::string(it->name()) == oms::ssp::Draft20180219::ssd::elements) // insert the parameter bindings after top-level connectors node
       {
-        pugi::xml_node node_parameters_bindings = node.insert_child_after(oms::ssp::Version1_0::ssd::parameter_bindings, *it);
+        pugi::xml_node node_parameters_bindings = node.insert_child_before(oms::ssp::Version1_0::ssd::parameter_bindings, *it);
         pugi::xml_node node_parameter_binding  = node_parameters_bindings.append_child(oms::ssp::Version1_0::ssd::parameter_binding);
         std::string ssvFileName = "resources/" + std::string(this->getCref()) + ".ssv";
         node_parameter_binding.append_attribute("source") = ssvFileName.c_str();
@@ -371,7 +574,8 @@ oms_status_enu_t oms::Model::exportToSSD(pugi::xml_node& node, pugi::xml_node& s
   pugi::xml_node node_annotations = default_experiment.append_child(oms::ssp::Draft20180219::ssd::annotations);
   pugi::xml_node node_annotation = node_annotations.append_child(oms::ssp::Version1_0::ssc::annotation);
   node_annotation.append_attribute("type") =  oms::ssp::Draft20180219::annotation_type;
-  pugi::xml_node oms_simulation_information = node_annotation.append_child(oms::ssp::Version1_0::simulation_information);
+  pugi::xml_node oms_annotation_node = node_annotation.append_child(oms::ssp::Version1_0::oms_annotations);
+  pugi::xml_node oms_simulation_information = oms_annotation_node.append_child(oms::ssp::Version1_0::simulation_information);
   //pugi::xml_node oms_default_experiment = oms_simulation_information.append_child("DefaultExperiment");
 
   oms_simulation_information.append_attribute("resultFile") = resultFilename.c_str();
@@ -430,7 +634,14 @@ oms_status_enu_t oms::Model::importFromSSD(const pugi::xml_node& node)
 
       if (annotation_node && std::string(annotation_node.attribute("type").as_string()) == oms::ssp::Draft20180219::annotation_type)
       {
-        for(pugi::xml_node_iterator itAnnotations = annotation_node.begin(); itAnnotations != annotation_node.end(); ++itAnnotations)
+        pugi::xml_node oms_annotation_node = annotation_node.child(oms::ssp::Version1_0::oms_annotations);
+        // support older <ssc:annotation>
+        if (!oms_annotation_node)
+        {
+          oms_annotation_node = annotation_node;
+          logWarning_deprecated;
+        }
+        for(pugi::xml_node_iterator itAnnotations = oms_annotation_node.begin(); itAnnotations != oms_annotation_node.end(); ++itAnnotations)
         {
           name = itAnnotations->name();
           // check for oms_default_experiment from version 1.0
@@ -439,7 +650,7 @@ oms_status_enu_t oms::Model::importFromSSD(const pugi::xml_node& node)
             resultFilename = itAnnotations->attribute("resultFile").as_string();
             loggingInterval = itAnnotations->attribute("loggingInterval").as_double();
             bufferSize = itAnnotations->attribute("bufferSize").as_int();
-            signalFilter = itAnnotations->attribute("signalFilter").as_string();
+            setSignalFilter(itAnnotations->attribute("signalFilter").as_string());
           }
         }
       }
@@ -453,7 +664,7 @@ oms_status_enu_t oms::Model::importFromSSD(const pugi::xml_node& node)
 
 oms_system_enu_t oms::Model::getSystemType(const pugi::xml_node& node, const std::string& sspVersion)
 {
-  oms_system_enu_t systemType;
+  oms_system_enu_t systemType = oms_system_none;
   for(pugi::xml_node_iterator itElements = node.begin(); itElements != node.end(); ++itElements)
   {
     std::string name = itElements->name();
@@ -478,7 +689,16 @@ oms_system_enu_t oms::Model::getSystemType(const pugi::xml_node& node, const std
 
       if (annotation_node && std::string(annotation_node.attribute("type").as_string()) == oms::ssp::Draft20180219::annotation_type)
       {
-        for(pugi::xml_node_iterator itAnnotations = annotation_node.begin(); itAnnotations != annotation_node.end(); ++itAnnotations)
+        pugi::xml_node oms_annotation_node;
+        oms_annotation_node = annotation_node.child(oms::ssp::Version1_0::oms_annotations);
+        // support older <ssc:annotation>
+        if (!oms_annotation_node)
+        {
+          oms_annotation_node = annotation_node;
+          logWarning_deprecated;
+        }
+
+        for(pugi::xml_node_iterator itAnnotations = oms_annotation_node.begin(); itAnnotations != oms_annotation_node.end(); ++itAnnotations)
         {
           std::string annotationName = itAnnotations->name();
           if (std::string(annotationName) == oms::ssp::Version1_0::simulation_information) // check for oms:simulationInformation from version 1.0
@@ -496,19 +716,19 @@ oms_system_enu_t oms::Model::getSystemType(const pugi::xml_node& node, const std
 oms_system_enu_t oms::Model::getSystemTypeHelper(const pugi::xml_node& node, const std::string& sspVersion)
 {
   oms_system_enu_t systemType = oms_system_tlm;
-  if (node.child(oms::ssp::Version1_0::VariableStepSolver).attribute("description").as_string() != "" || node.child("VariableStepSolver").attribute("description").as_string() !="")
+  if (std::string(node.child(oms::ssp::Version1_0::VariableStepSolver).attribute("description").as_string()) != "" || std::string(node.child("VariableStepSolver").attribute("description").as_string()) !="")
   {
     systemType = oms_system_sc;
   }
-  else if (node.child("oms:FixedStepSolver").attribute("description").as_string() != "" || node.child("FixedStepSolver").attribute("description").as_string() != "")
+  else if (std::string(node.child("oms:FixedStepSolver").attribute("description").as_string()) != "" || std::string(node.child("FixedStepSolver").attribute("description").as_string()) != "")
   {
     systemType = oms_system_sc;
   }
-  else if (node.child("oms:VariableStepMaster").attribute("description").as_string() != "" || node.child("VariableStepMaster").attribute("description").as_string() != "")
+  else if (std::string(node.child("oms:VariableStepMaster").attribute("description").as_string()) != "" || std::string(node.child("VariableStepMaster").attribute("description").as_string()) != "")
   {
     systemType = oms_system_wc;
   }
-  else if (node.child(oms::ssp::Version1_0::FixedStepMaster).attribute("description").as_string() != "" || node.child("FixedStepMaster").attribute("description").as_string() != "")
+  else if (std::string(node.child(oms::ssp::Version1_0::FixedStepMaster).attribute("description").as_string()) != "" || std::string(node.child("FixedStepMaster").attribute("description").as_string()) != "")
   {
     systemType = oms_system_wc;
   }
@@ -567,10 +787,10 @@ oms_status_enu_t oms::Model::exportToFile(const std::string& filename) const
     // update the ssd with the top level parameterBindings (e.g)  <ParameterBinding source="resources/ControlledTemperature.ssv">
     for(pugi::xml_node_iterator it = node.begin(); it != node.end(); ++it)
     {
-      pugi::xml_node node_connectors = it->child(oms::ssp::Draft20180219::ssd::connectors);
-      if (node_connectors) // insert the parameter bindings after top-level connectors node
+      pugi::xml_node node_elements = it->child(oms::ssp::Draft20180219::ssd::elements);
+      if (node_elements) // insert the parameter bindings before <ssd:Elements>
       {
-        pugi::xml_node node_parameters_bindings = it->insert_child_after(oms::ssp::Version1_0::ssd::parameter_bindings, node_connectors);
+        pugi::xml_node node_parameters_bindings = it->insert_child_before(oms::ssp::Version1_0::ssd::parameter_bindings, node_elements);
         pugi::xml_node node_parameter_binding  = node_parameters_bindings.append_child(oms::ssp::Version1_0::ssd::parameter_binding);
         node_parameter_binding.append_attribute("source") = ssvFileName.c_str();
         break;
@@ -952,15 +1172,21 @@ oms_status_enu_t oms::Model::getSignalFilter(char** regex)
   return oms_status_ok;
 }
 
-oms_status_enu_t oms::Model::setSignalFilter(const char* regex)
+oms_status_enu_t oms::Model::setSignalFilter(const std::string& regex)
 {
-  if (oms_status_ok != removeSignalsFromResults(".*"))
-    return oms_status_error;
+  // If regex is empty then all signals will be exported
+  if (regex.empty() || regex == ".*")
+    this->signalFilter = ".*";
+  else
+  {
+    if (oms_status_ok != removeSignalsFromResults(".*"))
+      return oms_status_error;
 
-  if (oms_status_ok != system->addSignalsToResults(regex))
-    return oms_status_error;
+    this->signalFilter = regex;
+  }
 
-  this->signalFilter = regex;
+  if (oms_status_ok != system->addSignalsToResults(this->signalFilter.c_str()))
+    return oms_status_error;
 
   return oms_status_ok;
 }
