@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -32,31 +32,35 @@
 #include "SystemTLM.h"
 
 #include "Component.h"
+#include "Flags.h"
 #include "Model.h"
+#include "OMTLMSimulatorLib/OMTLMSimulatorLib.h"
+#include "ssd/Tags.h"
 #include "SystemWC.h"
 #include "Types.h"
-#include "OMTLMSimulatorLib.h"
-#include "ssd/Tags.h"
-#include "OMTLMSimulatorLib.h"
 
 #include <thread>
 #include <algorithm>
 
-oms3::SystemTLM::SystemTLM(const ComRef& cref, Model* parentModel, System* parentSystem)
-  : oms3::System(cref, oms_system_tlm, parentModel, parentSystem)
+oms::SystemTLM::SystemTLM(const ComRef& cref, Model* parentModel, System* parentSystem)
+  : oms::System(cref, oms_system_tlm, parentModel, parentSystem, oms_solver_none)
 {
   logTrace();
   model = omtlm_newModel(cref.c_str());
-  omtlm_setLogLevel(model, 1);
-  omtlm_setNumLogStep(model, 1000);
+
+  omtlm_setLogLevel(model, TLMLogLevel::Fatal);
+  if(Log::getLoggingLevel() == 1)
+    omtlm_setLogLevel(model, TLMLogLevel::Info);
+  else if(Log::getLoggingLevel() == 2)
+    omtlm_setLogLevel(model, TLMLogLevel::Debug);
 }
 
-oms3::SystemTLM::~SystemTLM()
+oms::SystemTLM::~SystemTLM()
 {
   omtlm_unloadModel(model);
 }
 
-oms3::System* oms3::SystemTLM::NewSystem(const oms3::ComRef& cref, oms3::Model* parentModel, oms3::System* parentSystem)
+oms::System* oms::SystemTLM::NewSystem(const oms::ComRef& cref, oms::Model* parentModel, oms::System* parentSystem)
 {
   if (!cref.isValidIdent())
   {
@@ -74,16 +78,13 @@ oms3::System* oms3::SystemTLM::NewSystem(const oms3::ComRef& cref, oms3::Model*
   return system;
 }
 
-oms_status_enu_t oms3::SystemTLM::exportToSSD_SimulationInformation(pugi::xml_node& node) const
+oms_status_enu_t oms::SystemTLM::exportToSSD_SimulationInformation(pugi::xml_node& node) const
 {
-  pugi::xml_node node_simulation_information = node.append_child(oms2::ssd::ssd_simulation_information);
+  /* ssd:SimulationInformation should be added as vendor specific annotations from Version 1.0 */
+  pugi::xml_node node_simulation_information = node.append_child(oms::ssp::Version1_0::simulation_information);
 
-  pugi::xml_node node_annotations = node_simulation_information.append_child(oms2::ssd::ssd_annotations);
+  pugi::xml_node node_tlm = node_simulation_information.append_child(oms::ssp::Draft20180219::tlm_master);
 
-  pugi::xml_node node_annotation = node_annotations.append_child(oms2::ssd::ssd_annotation);
-  node_annotation.append_attribute("type") = oms::annotation_type;
-
-  pugi::xml_node node_tlm = node_annotation.append_child(oms::tlm_master);
   node_tlm.append_attribute("ip") = address.c_str();
   node_tlm.append_attribute("managerport") = std::to_string(desiredManagerPort).c_str();
   node_tlm.append_attribute("monitorport") = std::to_string(desiredMonitorPort).c_str();
@@ -91,30 +92,54 @@ oms_status_enu_t oms3::SystemTLM::exportToSSD_SimulationInformation(pugi::xml_no
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::importFromSSD_SimulationInformation(const pugi::xml_node& node)
+oms_status_enu_t oms::SystemTLM::importFromSSD_SimulationInformation(const pugi::xml_node& node, const std::string& sspVersion)
 {
-  pugi::xml_node annotationsNode = node.child(oms2::ssd::ssd_annotations);
-  if(annotationsNode) {
-    pugi::xml_node annotationNode = annotationsNode.child(oms2::ssd::ssd_annotation);
-    if(annotationNode && std::string(annotationNode.attribute("type").as_string()) == "org.openmodelica") {
-      pugi::xml_node tlmmasterNode = annotationNode.child("oms:TlmMaster");
-      for (auto it = tlmmasterNode.attributes_begin(); it != tlmmasterNode.attributes_end(); ++it)
-      {
-        std::string name = it->name();
-        if (name == "ip")
-          this->address = it->value();
-        else if(name == "managerport")
-          this->desiredManagerPort = tlmmasterNode.attribute("managerport").as_int();
-        else if(name == "monitorport")
-          this->desiredMonitorPort = tlmmasterNode.attribute("monitorport").as_int();
-      }
+  pugi::xml_node annotationsNode = node.child(oms::ssp::Draft20180219::ssd::annotations);
+
+  /*  To handle version = "Draft20180219"*/
+  if(annotationsNode)
+  {
+    pugi::xml_node annotationNode;
+    annotationNode = annotationsNode.child(oms::ssp::Version1_0::ssc::annotation);
+
+    // check for ssd:annotation to support older version, which is a bug
+    if(!annotationNode)
+    {
+      annotationNode = annotationsNode.child(oms::ssp::Draft20180219::ssd::annotation);
+      logWarning_deprecated;
     }
+
+    if(annotationNode && std::string(annotationNode.attribute("type").as_string()) == "org.openmodelica") {
+      importFromSSD_SimulationInformationHelper(annotationNode);
+    }
+  }
+  else
+  {
+    // for version = 1.0
+    importFromSSD_SimulationInformationHelper(node);
   }
 
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::instantiate()
+oms_status_enu_t oms::SystemTLM::importFromSSD_SimulationInformationHelper(const pugi::xml_node & node)
+{
+  pugi::xml_node tlmmasterNode = node.child("oms::ssp::Draft20180219::tlm_master");
+  for (auto it = tlmmasterNode.attributes_begin(); it != tlmmasterNode.attributes_end(); ++it)
+   {
+     std::string name = it->name();
+     if (name == "ip")
+       this->address = it->value();
+     else if(name == "managerport")
+       this->desiredManagerPort = tlmmasterNode.attribute("managerport").as_int();
+     else if(name == "monitorport")
+       this->desiredMonitorPort = tlmmasterNode.attribute("monitorport").as_int();
+   }
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::SystemTLM::instantiate()
 {
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->instantiate())
@@ -122,8 +147,10 @@ oms_status_enu_t oms3::SystemTLM::instantiate()
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::initialize()
+oms_status_enu_t oms::SystemTLM::initialize()
 {
+  omtlm_setLogStepSize(model, getModel()->getLoggingInterval());
+
   actualManagerPort = desiredManagerPort;
   actualMonitorPort = desiredMonitorPort;
 #ifndef _WIN32
@@ -136,8 +163,11 @@ oms_status_enu_t oms3::SystemTLM::initialize()
   omtlm_setMonitorPort(model, actualMonitorPort);
 
   for(const auto& subsystem : this->getSubSystems()) {
+    if(!singleModel.empty() && singleModel != subsystem.second->getCref().c_str()) {
+        continue;
+    }
     omtlm_addSubModel(model, subsystem.second->getCref().c_str(),"","none");
-    for(int i=0; subsystem.second->getTLMBusConnectors()[i]; ++i) {
+    for(int i=0; subsystem.second->getTLMBusConnectors()[i] && (singleModel==""); ++i) {
       TLMBusConnector *tlmbus = subsystem.second->getTLMBusConnectors()[i];
 
       std::string causality = "input";
@@ -160,6 +190,10 @@ oms_status_enu_t oms3::SystemTLM::initialize()
     if(component.second->getType() != oms_component_external)
       continue;
 
+    if(!singleModel.empty() && singleModel != component.second->getCref().c_str()) {
+        continue;
+    }
+
     ExternalModel* externalmodel = reinterpret_cast<ExternalModel*>(component.second);
 
     //Copy external model file to temporary directory
@@ -219,14 +253,17 @@ oms_status_enu_t oms3::SystemTLM::initialize()
 
   Connection** connections = this->getConnections(ComRef(""));
   for(int i=0; connections[i]; ++i) {
-    oms3_tlm_connection_parameters_t* tlmpars = connections[i]->getTLMParameters();
+    oms_tlm_connection_parameters_t* tlmpars = connections[i]->getTLMParameters();
     omtlm_addConnection(model,connections[i]->getSignalA().c_str(),connections[i]->getSignalB().c_str(),tlmpars->delay,tlmpars->linearimpedance,tlmpars->angularimpedance,tlmpars->alpha);
   }
 
+  logStep = (getModel()->getLoggingInterval() > 0) ? getModel()->getLoggingInterval() : 1e-3;
+  nextLogTime = getModel()->getStartTime()+logStep;
+
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::terminate()
+oms_status_enu_t oms::SystemTLM::terminate()
 {
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->terminate())
@@ -234,7 +271,7 @@ oms_status_enu_t oms3::SystemTLM::terminate()
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::reset()
+oms_status_enu_t oms::SystemTLM::reset()
 {
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->reset())
@@ -242,28 +279,25 @@ oms_status_enu_t oms3::SystemTLM::reset()
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::stepUntil(double stopTime, void (*cb)(const char* ident, double time, oms_status_enu_t status))
+oms_status_enu_t oms::SystemTLM::stepUntil(double stopTime, void (*cb)(const char* ident, double time, oms_status_enu_t status))
 {
   omtlm_setStartTime(model, getModel()->getStartTime());
   omtlm_setStopTime(model, stopTime);
 
   if(getSubSystems().empty() && getComponents().empty())
-    logWarning("oms3::TLMCompositeModel::stepUntil: Simulating empty model...");
+    logWarning("oms::TLMCompositeModel::stepUntil: Simulating empty model...");
 
   logInfo("Starting TLM manager in new thread");
 
   std::thread *masterThread = new std::thread(&omtlm_simulate, model);
 
-  logInfo("Connecting submodels to manager (threaded)");
+  logInfo("Connecting submodels to manager (sequential)");
   std::string server = address + ":" + std::to_string(actualManagerPort);
-  std::vector<std::thread> fmiConnectThreads;
   for(auto it = getSubSystems().begin(); it!=getSubSystems().end(); ++it) {
     System* subsystem = it->second;
     ComRef syscref = subsystem->getCref();
-    fmiConnectThreads.push_back(std::thread(&oms3::SystemTLM::connectToSockets, this, syscref, server));
+    connectToSockets(syscref, server);
   }
-  for(auto &thread : fmiConnectThreads)
-    thread.join();
   for(auto it = getSubSystems().begin(); it!=getSubSystems().end(); ++it) {
     if(find(connectedsubsystems.begin(), connectedsubsystems.end(), it->second->getCref()) == connectedsubsystems.end())
       return logError("Failed to connect TLM subsystem: "+std::string(it->second->getCref()));
@@ -274,7 +308,7 @@ oms_status_enu_t oms3::SystemTLM::stepUntil(double stopTime, void (*cb)(const ch
   for(auto it = getSubSystems().begin(); it!=getSubSystems().end(); ++it) {
     System *subsystem = it->second;
     ComRef syscref = subsystem->getCref();
-    fmiInitializeThreads.push_back(std::thread(&oms3::SystemTLM::initializeSubSystem, this, syscref));
+    fmiInitializeThreads.push_back(std::thread(&oms::SystemTLM::initializeSubSystem, this, syscref));
   }
   for(auto &thread : fmiInitializeThreads)
     thread.join();
@@ -288,7 +322,7 @@ oms_status_enu_t oms3::SystemTLM::stepUntil(double stopTime, void (*cb)(const ch
   for(auto it = getSubSystems().begin(); it!=getSubSystems().end(); ++it) {
     System* subsystem = it->second;
     ComRef syscref = subsystem->getCref();
-    fmiSimulateThreads.push_back(std::thread(&oms3::SystemTLM::simulateSubSystem, this, syscref, stopTime));
+    fmiSimulateThreads.push_back(std::thread(&oms::SystemTLM::simulateSubSystem, this, syscref, stopTime));
   }
   for(auto &thread : fmiSimulateThreads)
     thread.join();
@@ -300,7 +334,7 @@ oms_status_enu_t oms3::SystemTLM::stepUntil(double stopTime, void (*cb)(const ch
   std::vector<std::thread> fmiDisconnectThreads;
   for(auto it = getSubSystems().begin(); it!=getSubSystems().end(); ++it) {
     System* subsystem = it->second;
-    fmiDisconnectThreads.push_back(std::thread(&oms3::SystemTLM::disconnectFromSockets, this, subsystem->getCref()));
+    fmiDisconnectThreads.push_back(std::thread(&oms::SystemTLM::disconnectFromSockets, this, subsystem->getCref()));
   }
   for(auto &thread : fmiDisconnectThreads)
     thread.join();
@@ -310,7 +344,7 @@ oms_status_enu_t oms3::SystemTLM::stepUntil(double stopTime, void (*cb)(const ch
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::connectToSockets(const oms3::ComRef cref, std::string server)
+oms_status_enu_t oms::SystemTLM::connectToSockets(const oms::ComRef cref, std::string server)
 {
   logInfo("Starting TLM simulation thread for model "+std::string(cref));
 
@@ -324,9 +358,9 @@ oms_status_enu_t oms3::SystemTLM::connectToSockets(const oms3::ComRef cref, std:
   TLMBusConnector** tlmbuses = system->getTLMBusConnectors();
   for (int i=0; tlmbuses[i]; ++i)
   {
-      if(system->getStepSize() > tlmbuses[i]->getDelay()*0.5) {
+      if(system->getMaximumStepSize() > tlmbuses[i]->getDelay()*0.5) {
         system->setFixedStepSize(tlmbuses[i]->getDelay()*0.5);
-        logInfo("Limiting stepSize for "+std::string(getCref())+"."+std::string(tlmbuses[i]->getName())+" to "+std::to_string(system->getStepSize()));
+        logInfo("Limiting stepSize for "+std::string(getCref())+"."+std::string(tlmbuses[i]->getName())+" to "+std::to_string(system->getMaximumStepSize()));
       }
   }
 
@@ -335,24 +369,25 @@ oms_status_enu_t oms3::SystemTLM::connectToSockets(const oms3::ComRef cref, std:
   TLMPlugin* plugin = TLMPlugin::CreateInstance();
   pluginsMutex.lock();
   plugins[system] = plugin;
-  pluginsMutex.unlock();
 
   logInfo("Initializing plugin for "+std::string(cref));
 
   if(!plugin->Init(std::string(cref),
                    getModel()->getStartTime(),
                    1, //Unused argument anyway
-                   system->getStepSize(),
+                   system->getMaximumStepSize(),
                    server)) {
     logError("Error initializing the TLM plugin for "+std::string(cref));
+    pluginsMutex.unlock();
     return oms_status_error;
   }
+  pluginsMutex.unlock();
 
   logInfo("Registering interfaces for "+std::string(cref));
 
   for (int i=0; tlmbuses[i]; ++i)
   {
-    TLMBusConnector* bus = tlmbuses[i];
+    TLMBusConnector* bus = tlmbuses[i]->getActualBus();
     oms_status_enu_t status = bus->registerToSockets(plugin);
     if(status == oms_status_error) {
       return logError("Failed to register interface "+std::string(bus->getName()));
@@ -366,7 +401,7 @@ oms_status_enu_t oms3::SystemTLM::connectToSockets(const oms3::ComRef cref, std:
   return oms_status_ok;
 }
 
-void oms3::SystemTLM::disconnectFromSockets(const oms3::ComRef cref)
+void oms::SystemTLM::disconnectFromSockets(const oms::ComRef cref)
 {
   SystemWC* system = reinterpret_cast<SystemWC*>(getSystem(cref));
   if(system != nullptr)
@@ -381,9 +416,9 @@ void oms3::SystemTLM::disconnectFromSockets(const oms3::ComRef cref)
   }
 }
 
-oms_status_enu_t oms3::SystemTLM::setSocketData(const std::string &address, int managerPort, int monitorPort)
+oms_status_enu_t oms::SystemTLM::setSocketData(const std::string &address, int managerPort, int monitorPort)
 {
-  if (oms_modelState_terminated != getModel()->getModelState())
+  if (!getModel()->validState(oms_modelState_virgin))
     return logError_ModelInWrongState(this);
 
   this->address = address;
@@ -392,7 +427,7 @@ oms_status_enu_t oms3::SystemTLM::setSocketData(const std::string &address, int
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::setPositionAndOrientation(const oms3::ComRef &cref, std::vector<double> x, std::vector<double> A)
+oms_status_enu_t oms::SystemTLM::setPositionAndOrientation(const oms::ComRef &cref, std::vector<double> x, std::vector<double> A)
 {
   ComRef tail = cref;
   ComRef head = tail.pop_front();
@@ -412,7 +447,47 @@ oms_status_enu_t oms3::SystemTLM::setPositionAndOrientation(const oms3::ComRef &
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::updateInitialValues(const oms3::ComRef cref)
+oms_status_enu_t oms::SystemTLM::fetchInterfaces(const oms::ComRef &cref,
+                                                 std::vector<std::string> &names,
+                                                 std::vector<int> &dimensions,
+                                                 std::vector<std::string> &domains)
+{
+    if(getComponents().find(cref) == getComponents().end()) {
+      return logError("Sub-model \""+std::string(cref)+"\" not found.");
+    }
+    singleModel = cref.c_str();
+    initialize();
+    singleModel = "";
+    omtlm_fetchInterfaces(model, cref.c_str());
+
+    //Read interface data from interfaceData.xml (generated above)
+    pugi::xml_document doc;
+    pugi::xml_parse_result result = doc.load_file("interfaceData.xml");
+    if(!result) {
+        return logError("reading interface data from interfaceData.xml failed");
+    }
+
+    pugi::xml_node modelDataNode = doc.child("ModelData");
+    if(!modelDataNode) {
+        return logError("\"ModelData\" node not found in interfaceData.xml");
+    }
+
+    pugi::xml_node interfacesNode = modelDataNode.child("Interfaces");
+    if(!interfacesNode) {
+        return logError("\"Interfaces\" node not found in interfaceData.xml");
+    }
+
+    for (pugi::xml_node interfaceNode = interfacesNode.child("Interface"); interfaceNode; interfaceNode = interfaceNode.next_sibling("Interface"))
+    {
+        names.push_back(interfaceNode.attribute("Name").value());
+        dimensions.push_back(interfaceNode.attribute("Dimensions").as_int());
+        domains.push_back(interfaceNode.attribute("Domain").value());
+    }
+
+    return oms_status_ok;
+}
+
+oms_status_enu_t oms::SystemTLM::updateInitialValues(const oms::ComRef cref)
 {
   SystemWC* system = reinterpret_cast<SystemWC*>(getSystem(cref));
   if(system == nullptr)
@@ -426,19 +501,20 @@ oms_status_enu_t oms3::SystemTLM::updateInitialValues(const oms3::ComRef cref)
   for (int i=0; tlmbuses[i]; ++i)
   {
     TLMBusConnector* bus = tlmbuses[i];
+    bus = bus->getActualBus();
 
     if(bus->getDimensions() == 1 && bus->getCausality() == oms_causality_input) {
       oms_tlm_sigrefs_signal_t tlmrefs;
       double value;
-      system->getReal(bus->getConnector(tlmrefs.y), value);
+      bus->getReal(tlmrefs.y, value);
       plugin->SetInitialValue(bus->getId(), value);
     }
     else if(bus->getDimensions() == 1 && bus->getCausality() == oms_causality_bidir &&
             bus->getInterpolation() == oms_tlm_coarse_grained) {
       oms_tlm_sigrefs_1d_cg_t tlmrefs;
       double effort,flow;
-      system->getReal(bus->getConnector(tlmrefs.c), effort);
-      system->getReal(bus->getConnector(tlmrefs.v), flow);
+      bus->getReal(tlmrefs.c, effort);
+      bus->getReal(tlmrefs.v, flow);
       plugin->SetInitialForce1D(bus->getId(), effort);
       plugin->SetInitialFlow1D(bus->getId(), flow);
     }
@@ -446,8 +522,8 @@ oms_status_enu_t oms3::SystemTLM::updateInitialValues(const oms3::ComRef cref)
             bus->getInterpolation() == oms_tlm_fine_grained) {
       oms_tlm_sigrefs_1d_fg_t tlmrefs;
       double effort,flow;
-      system->getReal(bus->getConnector(tlmrefs.c[0]), effort);
-      system->getReal(bus->getConnector(tlmrefs.v), flow);
+      bus->getReal(tlmrefs.c[0], effort);
+      bus->getReal(tlmrefs.v, flow);
       plugin->SetInitialForce1D(bus->getId(), effort);
       plugin->SetInitialFlow1D(bus->getId(), flow);
     }
@@ -455,12 +531,11 @@ oms_status_enu_t oms3::SystemTLM::updateInitialValues(const oms3::ComRef cref)
             bus->getInterpolation() == oms_tlm_no_interpolation) {
       oms_tlm_sigrefs_3d_t tlmrefs;
       std::vector<double> effort(6,0);
-      std::vector<double> flow(3,0);
-      std::vector<double> angularflow(3,0);
-      system->getReals(bus->getConnectors(tlmrefs.f), effort);
+      std::vector<double> flow(6,0);
+      bus->getReals(tlmrefs.f, effort);
       std::vector<int> flowrefs = tlmrefs.v;
       flowrefs.insert(flowrefs.end(), tlmrefs.w.begin(), tlmrefs.w.end());
-      system->getReals(bus->getConnectors(flowrefs), flow);
+      bus->getReals(flowrefs, flow);
       plugin->SetInitialForce3D(bus->getId(), effort[0], effort[1], effort[2], effort[3], effort[4], effort[5]);
       plugin->SetInitialFlow3D(bus->getId(), flow[0], flow[1], flow[2], flow[3], flow[4], flow[5]);
     }
@@ -468,12 +543,11 @@ oms_status_enu_t oms3::SystemTLM::updateInitialValues(const oms3::ComRef cref)
             bus->getInterpolation() == oms_tlm_coarse_grained) {
       oms_tlm_sigrefs_3d_cg_t tlmrefs;
       std::vector<double> effort(6,0);
-      std::vector<double> flow(3,0);
-      std::vector<double> angularflow(3,0);
-      system->getReals(bus->getConnectors(tlmrefs.c), effort);
+      std::vector<double> flow(6,0);
+      bus->getReals(tlmrefs.c, effort);
       std::vector<int> flowrefs = tlmrefs.v;
       flowrefs.insert(flowrefs.end(), tlmrefs.w.begin(), tlmrefs.w.end());
-      system->getReals(bus->getConnectors(flowrefs), flow);
+      bus->getReals(flowrefs, flow);
       plugin->SetInitialForce3D(bus->getId(), effort[0], effort[1], effort[2], effort[3], effort[4], effort[5]);
       plugin->SetInitialFlow3D(bus->getId(), flow[0], flow[1], flow[2], flow[3], flow[4], flow[5]);
     }
@@ -481,12 +555,11 @@ oms_status_enu_t oms3::SystemTLM::updateInitialValues(const oms3::ComRef cref)
             bus->getInterpolation() == oms_tlm_fine_grained) {
       oms_tlm_sigrefs_3d_fg_t tlmrefs;
       std::vector<double> effort(6,0);
-      std::vector<double> flow(3,0);
-      std::vector<double> angularflow(3,0);
-      system->getReals(bus->getConnectors(tlmrefs.c[0]), effort);
+      std::vector<double> flow(6,0);
+      bus->getReals(tlmrefs.c[0], effort);
       std::vector<int> flowrefs = tlmrefs.v;
       flowrefs.insert(flowrefs.end(), tlmrefs.w.begin(), tlmrefs.w.end());
-      system->getReals(bus->getConnectors(flowrefs), flow);
+      bus->getReals(flowrefs, flow);
       plugin->SetInitialForce3D(bus->getId(), effort[0], effort[1], effort[2], effort[3], effort[4], effort[5]);
       plugin->SetInitialFlow3D(bus->getId(), flow[0], flow[1], flow[2], flow[3], flow[4], flow[5]);
     }
@@ -495,12 +568,13 @@ oms_status_enu_t oms3::SystemTLM::updateInitialValues(const oms3::ComRef cref)
   return oms_status_ok;
 }
 
-oms_status_enu_t oms3::SystemTLM::initializeSubSystem(oms3::ComRef cref)
+oms_status_enu_t oms::SystemTLM::initializeSubSystem(oms::ComRef cref)
 {
   oms_status_enu_t status = getSubSystem(cref)->initialize();
-  if(status = oms_status_ok)
-    status = updateInitialValues(cref);
-  if(status == oms_status_ok) {
+  if(oms_status_ok != status)
+    return status;
+  status = updateInitialValues(cref);
+  if(oms_status_ok == status) {
     setInitializedMutex.lock();
     initializedsubsystems.push_back(cref);
     setInitializedMutex.unlock();
@@ -508,14 +582,14 @@ oms_status_enu_t oms3::SystemTLM::initializeSubSystem(oms3::ComRef cref)
   return status;
 }
 
-oms_status_enu_t oms3::SystemTLM::simulateSubSystem(oms3::ComRef cref, double stopTime)
+oms_status_enu_t oms::SystemTLM::simulateSubSystem(oms::ComRef cref, double stopTime)
 {
   oms_status_enu_t status = getSubSystem(cref)->stepUntil(stopTime, NULL);
   plugins[getSubSystem(cref)]->AwaitClosePermission();
   return status;
 }
 
-void oms3::SystemTLM::writeToSockets(SystemWC *system, double time, Component* component)
+void oms::SystemTLM::writeToSockets(SystemWC *system, double time, Component* component)
 {
   socketMutexes[system].lock();
 
@@ -524,8 +598,8 @@ void oms3::SystemTLM::writeToSockets(SystemWC *system, double time, Component* c
   for(int i=0; tlmbuses[i]; ++i)
   {
     TLMBusConnector* bus = tlmbuses[i];
-    int id = bus->getId();      //Need socket ID from top-level bus
     bus = bus->getActualBus();  //Communicate with actual bus
+    int id = bus->getId();
 
     if(component && component != bus->getComponent()) {
       continue; //Ignore FMUs not specified in vector
@@ -535,8 +609,9 @@ void oms3::SystemTLM::writeToSockets(SystemWC *system, double time, Component* c
     if(bus->getDimensions() == 1 && bus->getCausality() == oms_causality_output) {
       oms_tlm_sigrefs_signal_t tlmrefs;
       double value;
-      system->getReal(bus->getConnector(tlmrefs.y), value);
+      bus->getReal(tlmrefs.y, value);
       plugin->SetValueSignal(id, time, value);
+      sendValueToLogger(busLogIds[bus], time, value);
     }
     else if(bus->getDimensions() == 1 && bus->getCausality() == oms_causality_bidir) {
       oms_tlm_sigrefs_1d_t tlmrefs;
@@ -550,6 +625,9 @@ void oms3::SystemTLM::writeToSockets(SystemWC *system, double time, Component* c
 
       //Send the resulting motion back to master
       plugin->SetMotion1D(id, time, state, flow);
+      sendValueToLogger(busLogIds[bus], time, state);
+      sendValueToLogger(busLogIds[bus]+1, time, flow);
+      sendValueToLogger(busLogIds[bus]+2, time, force);
     }
     else if(bus->getDimensions() == 3 && bus->getCausality() == oms_causality_bidir) {
 
@@ -561,10 +639,10 @@ void oms3::SystemTLM::writeToSockets(SystemWC *system, double time, Component* c
       std::vector<double> w(3,0);
       std::vector<double> f(6,0);
 
-      system->getReals(bus->getConnectors(tlmrefs.x), x);
-      system->getReals(bus->getConnectors(tlmrefs.A), A);
-      system->getReals(bus->getConnectors(tlmrefs.v), v);
-      system->getReals(bus->getConnectors(tlmrefs.w), w);
+      bus->getReals(tlmrefs.x, x);
+      bus->getReals(tlmrefs.A, A);
+      bus->getReals(tlmrefs.v, v);
+      bus->getReals(tlmrefs.w, w);
 
       //Important: OMTLMSimulator assumes that GetForce is called
       //before SetMotion, in order to calculate the wave variable
@@ -572,13 +650,40 @@ void oms3::SystemTLM::writeToSockets(SystemWC *system, double time, Component* c
 
       //Send the resulting motion back to master
       plugin->SetMotion3D(id, time, &x[0], &A[0], &v[0], &w[0]);
+
+      sendValueToLogger(busLogIds[bus], time, x[0]);
+      sendValueToLogger(busLogIds[bus]+1, time, x[1]);
+      sendValueToLogger(busLogIds[bus]+2, time, x[2]);
+      sendValueToLogger(busLogIds[bus]+3, time, A[0]);
+      sendValueToLogger(busLogIds[bus]+4, time, A[1]);
+      sendValueToLogger(busLogIds[bus]+5, time, A[2]);
+      sendValueToLogger(busLogIds[bus]+6, time, A[3]);
+      sendValueToLogger(busLogIds[bus]+7, time, A[4]);
+      sendValueToLogger(busLogIds[bus]+8, time, A[5]);
+      sendValueToLogger(busLogIds[bus]+9, time, A[6]);
+      sendValueToLogger(busLogIds[bus]+10, time, A[7]);
+      sendValueToLogger(busLogIds[bus]+11, time, A[8]);
+      sendValueToLogger(busLogIds[bus]+12, time, v[0]);
+      sendValueToLogger(busLogIds[bus]+13, time, v[1]);
+      sendValueToLogger(busLogIds[bus]+14, time, v[2]);
+      sendValueToLogger(busLogIds[bus]+15, time, w[0]);
+      sendValueToLogger(busLogIds[bus]+16, time, w[1]);
+      sendValueToLogger(busLogIds[bus]+17, time, w[2]);
+      sendValueToLogger(busLogIds[bus]+18, time, f[0]);
+      sendValueToLogger(busLogIds[bus]+19, time, f[1]);
+      sendValueToLogger(busLogIds[bus]+20, time, f[2]);
+      sendValueToLogger(busLogIds[bus]+21, time, f[3]);
+      sendValueToLogger(busLogIds[bus]+22, time, f[4]);
+      sendValueToLogger(busLogIds[bus]+23, time, f[5]);
     }
   }
 
+  sendValuesToLogger(system, time);
+
   socketMutexes[system].unlock();
 }
 
-void oms3::SystemTLM::readFromSockets(SystemWC* system, double time, Component* component)
+void oms::SystemTLM::readFromSockets(SystemWC* system, double time, Component* component)
 {
   socketMutexes[system].lock();
 
@@ -587,8 +692,8 @@ void oms3::SystemTLM::readFromSockets(SystemWC* system, double time, Component*
   for(int i=0; tlmbuses[i]; ++i)
   {
     TLMBusConnector* bus = tlmbuses[i];
-    int id = bus->getId();      //Need socket ID from top-level bus
     bus = bus->getActualBus();  //Communicate with actual bus
+    int id = bus->getId();
 
     if(component && component != bus->getComponent()) {
       continue; //Ignore FMUs not specified in vector
@@ -631,9 +736,9 @@ void oms3::SystemTLM::readFromSockets(SystemWC* system, double time, Component*
       bus->setReal(tlmrefs.Z, impedance);
 
       double impedance2, wave2;
-      plugin->GetWaveImpedance1D(id, time+system->getStepSize(), &impedance2, &wave2);
+      plugin->GetWaveImpedance1D(id, time+system->getMaximumStepSize(), &impedance2, &wave2);
 
-      double dWave = (wave2-wave)/system->getStepSize();
+      double dWave = (wave2-wave)/system->getMaximumStepSize();
 
       bus->setRealInputDerivatives(tlmrefs.c, 1, dWave);
     }
@@ -647,7 +752,7 @@ void oms3::SystemTLM::readFromSockets(SystemWC* system, double time, Component*
       double t = time;
       for(size_t i=0; i<10; ++i) {
         plugin->GetWaveImpedance1D(id, t, &impedance, &wave);
-        t += system->getStepSize()/9;
+        t += system->getMaximumStepSize()/9;
 
         bus->setReal(tlmrefs.c[i], wave);
         bus->setReal(tlmrefs.t[i], t);
@@ -692,11 +797,11 @@ void oms3::SystemTLM::readFromSockets(SystemWC* system, double time, Component*
 
       std::vector<double> waves2(6,0);
       double Zt2, Zr2;
-      plugin->GetWaveImpedance3D(id, time+system->getStepSize(), &Zt2, &Zr2, &waves2[0]);
+      plugin->GetWaveImpedance3D(id, time+system->getMaximumStepSize(), &Zt2, &Zr2, &waves2[0]);
 
       std::vector<double> dWaves(6,0);
       for(size_t i=0; i<6; ++i) {
-        double dWave = (waves2[i]-waves[i])/system->getStepSize();
+        double dWave = (waves2[i]-waves[i])/system->getMaximumStepSize();
         bus->setRealInputDerivatives(tlmrefs.c[i], 1, dWave);
       }
     }
@@ -711,7 +816,7 @@ void oms3::SystemTLM::readFromSockets(SystemWC* system, double time, Component*
 
       for(size_t i=0; i<10; ++i) {
         plugin->GetWaveImpedance3D(id, t, &Zt, &Zr, &waves[0]);
-        t += system->getStepSize()/9;
+        t += system->getMaximumStepSize()/9;
 
         bus->setReals(tlmrefs.c[i], waves);
         bus->setReal(tlmrefs.t[i], t);
@@ -724,3 +829,182 @@ void oms3::SystemTLM::readFromSockets(SystemWC* system, double time, Component*
 
   socketMutexes[system].unlock();
 }
+
+
+void oms::SystemTLM::sendValueToLogger(int varId, double time, double value)
+{
+  logMutex.lock();
+  logBuffer[varId].push_back(std::make_pair(time, value));
+
+  double tmax = 1e20;
+  for(auto& varbuffer : logBuffer) {
+    if(varbuffer.second.back().first < tmax) {
+      tmax = varbuffer.second.back().first;
+    }
+  }
+
+  if(tmax >= nextLogTime && logBuffer.size() == numLogVars)
+  {
+    logTime = nextLogTime;
+    getModel()->emit(nextLogTime);
+    nextLogTime += logStep;
+  }
+
+  logMutex.unlock();
+}
+
+oms_status_enu_t oms::SystemTLM::registerSignalsForResultFile(ResultWriter& resultFile)
+{
+  if (Flags::WallTime())
+    clock_id = resultFile.addSignal(std::string(getFullCref() + ComRef("$wallTime")), "wall-clock time [s]", SignalType_REAL);
+  else
+    clock_id = 0;
+
+  for(auto& system : getSubSystems())
+  {
+    TLMBusConnector** tlmbuses = system.second->getTLMBusConnectors();
+    for(int i=0; tlmbuses[i]; ++i)
+    {
+      TLMBusConnector* bus = tlmbuses[i];
+      bus = bus->getActualBus();  //Communicate with actual bus
+      if(bus->getDimensions() == 1 && bus->getCausality() == oms_causality_output) {
+        busLogIds[bus] = registerLogVariable();
+        resultFile.addSignal(std::string(system.second->getFullCref()+bus->getName()), "", SignalType_REAL);
+      }
+      else if(bus->getDimensions() == 1 && bus->getCausality() == oms_causality_bidir) {
+        std::string statename,flowname,effortname;
+        if(bus->getDomain() == oms_tlm_domain_mechanical) {
+          statename = "x";
+          flowname =  "v";
+          effortname = "f";
+        }
+        else if(bus->getDomain() == oms_tlm_domain_electric) {
+          statename = "phi";
+          flowname =  "w";
+          effortname = "T";
+        }
+        else if(bus->getDomain() == oms_tlm_domain_hydraulic) {
+          statename = "V";
+          flowname =  "q";
+          effortname = "p";
+        }
+        else if(bus->getDomain() == oms_tlm_domain_electric) {
+          statename = "Q";
+          flowname =  "i";
+          effortname = "U";
+        }
+        busLogIds[bus] = registerLogVariable();
+        resultFile.addSignal(std::string(system.second->getFullCref()+bus->getName())+"."+statename, "", SignalType_REAL);
+        registerLogVariable();
+        resultFile.addSignal(std::string(system.second->getFullCref()+bus->getName())+"."+flowname, "", SignalType_REAL);
+        registerLogVariable();
+        resultFile.addSignal(std::string(system.second->getFullCref()+bus->getName())+"."+effortname, "", SignalType_REAL);
+      }
+      else if(bus->getDimensions() == 3 && bus->getCausality() == oms_causality_bidir) {
+        busLogIds[bus] = registerLogVariable();
+        resultFile.addSignal(std::string(system.second->getFullCref()+bus->getName())+".x[1]", "", SignalType_REAL);
+        std::vector<std::string> varNames = {"x[2]","x[3]",
+                                             "A[1,1]","A[1,2]","A[1,3]","A[2,1]","A[2,2]","A[2,3]","A[3,1]","A[3,2]","A[3,3]",
+                                             "v[1]","v[2]","v[3]","w[1]","w[2]","w[3]",
+                                             "f[1]","f[2]","f[3]","t[1]","t[2]","t[3]"};
+        for(std::string& varName : varNames) {
+          registerLogVariable();
+          resultFile.addSignal(std::string(system.second->getFullCref()+bus->getName())+"."+varName, "", SignalType_REAL);
+        }
+      }
+    }
+    registerLogVariables(system.second, resultFile);
+  }
+
+  return oms_status_ok;
+}
+
+int oms::SystemTLM::registerLogVariable()
+{
+  return numLogVars++;
+}
+
+void oms::SystemTLM::registerLogVariables(oms::System *system, ResultWriter& resultFile)
+{
+  for(int i=0; system->getConnectors()[i]; ++i) {
+    if(system->getConnectors()[i]->isOutput() && system->getConnectors()[i]->isTypeReal()) {
+      connectorLogIds[system->getConnectors()[i]] = registerLogVariable();
+      resultFile.addSignal(std::string(system->getFullCref()+system->getConnectors()[i]->getName()), "", SignalType_REAL);
+    }
+  }
+
+  for(auto& component : system->getComponents()) {
+    for(int i=0; component.second->getConnectors()[i]; ++i) {
+      if(component.second->getConnectors()[i]->isOutput() && component.second->getConnectors()[i]->isTypeReal()) {
+        connectorLogIds[component.second->getConnectors()[i]] = registerLogVariable();
+        resultFile.addSignal(std::string(system->getFullCref()+component.second->getCref()+component.second->getConnectors()[i]->getName()), "", SignalType_REAL);
+      }
+    }
+  }
+
+  for(auto& subsystem : system->getSubSystems()) {
+    registerLogVariables(subsystem.second, resultFile);
+  }
+}
+
+void oms::SystemTLM::sendValuesToLogger(oms::System *system, double time)
+{
+  for(int i=0; system->getConnectors()[i]; ++i) {
+    if(system->getConnectors()[i]->isOutput() && system->getConnectors()[i]->isTypeReal()) {
+      double value;
+      system->getReal(system->getConnectors()[i]->getName(), value);
+      sendValueToLogger(connectorLogIds[system->getConnectors()[i]], time, value);
+    }
+  }
+
+  for(auto& component : system->getComponents()) {
+    for(int i=0; component.second->getConnectors()[i]; ++i) {
+      if(component.second->getConnectors()[i]->isOutput() && component.second->getConnectors()[i]->isTypeReal()) {
+        double value;
+        component.second->getReal(component.second->getConnectors()[i]->getName(), value);
+        sendValueToLogger(connectorLogIds[component.second->getConnectors()[i]], time, value);
+      }
+    }
+  }
+
+  for(auto& subsystem : system->getSubSystems()) {
+    sendValuesToLogger(subsystem.second, time);
+  }
+}
+
+oms_status_enu_t oms::SystemTLM::updateSignals(oms::ResultWriter &resultFile)
+{
+  if (logBuffer.size() == 0)
+    return oms_status_ok;
+
+  int i = (clock_id) ? 2 : 1;  // TODO: see #596
+
+  if (clock_id)
+  {
+    SignalValue_t wallTime;
+    wallTime.realValue = clock.getElapsedWallTime();
+    resultFile.updateSignal(clock_id, wallTime);
+  }
+
+  for(auto& varbuffer : logBuffer) {
+    auto& databuffer = varbuffer.second;
+    double x1,x2,t1,t2;
+    t2 = databuffer.back().first;
+    x2 = databuffer.back().second;
+    t1 = t2;
+    x1 = x2;
+    for(int i=databuffer.size()-1; i>=0; --i) {
+      if(databuffer[i].first < nextLogTime) {
+        t1 = databuffer[i].first;
+        x1 = databuffer[i].second;
+        break;
+      }
+    }
+
+    SignalValue_t value;
+    value.realValue = interpolate(t1,t2,x1,x2,logTime);
+    resultFile.updateSignal(i, value);
+    ++i;
+  }
+  return oms_status_ok;
+}
